@@ -5,10 +5,9 @@
  *
  * Author:  jacob berkman  <jacob@bug-buddy.org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,17 +21,20 @@
 
 #include <config.h>
 
+#include "bug-buddy.h"
+#include "libglade-buddy.h"
+
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <utime.h>
 #include <errno.h>
+#include <string.h>
+#include <ctype.h>
 
 #include <gnome.h>
 
 #include <libgnomevfs/gnome-vfs.h>
 
-#include "bug-buddy.h"
-#include "libglade-buddy.h"
 #include <dirent.h>
 
 #include <libxml/tree.h>
@@ -72,9 +74,13 @@ load_config_xml (BugzillaBTS *bts, xmlDoc *doc)
 	xmlNode *node, *cur;
 	char *s;
 
-	for (node = doc->root->childs; node; node = node->next) {
+	d(g_print ("config:\n"));
+
+	for (node = xmlDocGetRootElement (doc)->children; node; node = node->next) {
+		d(g_print ("\t%s\n", node->name));
 		if (!strcmp (node->name, bts->severity_node)) {
-			for (cur = node->childs; cur; cur = cur->next) {
+			for (cur = node->children; cur; cur = cur->next) {
+				d(g_print ("\t\t%s\n", cur->name));
 				if (strcmp (cur->name, bts->severity_item))
 					continue;
 				s = xmlNodeGetContent (cur);
@@ -83,7 +89,8 @@ load_config_xml (BugzillaBTS *bts, xmlDoc *doc)
 				xmlFree (s);
 			}
 		} else if (!strcmp (node->name, "opsys_list")) {
-			for (cur = node->childs; cur; cur = cur->next) {
+			for (cur = node->children; cur; cur = cur->next) {
+				d(g_print ("\t\t%s\n", cur->name));
 				if (strcmp (cur->name, "opsys"))
 					continue;
 				s = xmlNodeGetContent (cur);
@@ -105,7 +112,10 @@ load_products_xml (BugzillaBTS *bts, xmlDoc *doc)
 	xmlNode *node, *cur;
 	char *s;
 
-	for (node = doc->root->childs; node; node = node->next) {
+	d(g_print ("products:\n"));
+
+	for (node = xmlDocGetRootElement (doc)->children; node; node = node->next) {
+		d(g_print ("\t%s\n", node->name));
 		if (!strcmp (node->name, "product")) {
 			prod = g_new0 (BugzillaProduct, 1);
 			prod->bts = bts;
@@ -121,7 +131,8 @@ load_products_xml (BugzillaBTS *bts, xmlDoc *doc)
 			bugzilla_bts_insert_product (bts, prod);
 			bugzilla_bts_insert_product (druid_data.all_products, prod);
 
-			for (cur = node->childs; cur; cur = cur->next) {
+			for (cur = node->children; cur; cur = cur->next) {
+				d(g_print ("\t\t%s\n", node->name));
 				if (strcmp (cur->name, "component"))
 					continue;
 				
@@ -183,16 +194,17 @@ async_update (GnomeVFSAsyncHandle *handle, GnomeVFSXferProgressInfo *info, gpoin
 	d(g_print ("%lu\n", info->bytes_copied));
 	if (info->source_name) {
 		d(g_print ("source: %s\n", info->source_name));
-		gtk_label_set_text (GTK_LABEL (GET_WIDGET ("progress-source")), info->source_name);
+		buddy_set_text ("progress-source", info->source_name);
 	}
 
 	if (info->target_name) {
 		d(g_print ("target: %s\n", info->target_name));
-		gtk_label_set_text (GTK_LABEL (GET_WIDGET ("progress-dest")), info->target_name);
+		buddy_set_text ("progress-dest", info->target_name);
 	}
 
 	if (info->bytes_total)
-		gtk_progress_set_percentage (GTK_PROGRESS (GET_WIDGET ("progress-progress")), (gfloat)info->total_bytes_copied / info->bytes_total);
+		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (GET_WIDGET ("progress-progress")), 
+					       (gfloat)info->total_bytes_copied / info->bytes_total);
 
 	if (info->phase == GNOME_VFS_XFER_PHASE_COMPLETED) {
 		GList *li;
@@ -272,7 +284,6 @@ get_xml_file (BugzillaBTS *bts, const char *key, XMLFunc parse_func)
 {
 	BugzillaXMLFile *xmlfile;
 	char *localdir, *tmppath, *src_uri;
-	char *err;
 	struct stat sys_stat, local_stat;
 	gboolean sys_is_newer, cache_is_old;
 	xmlDoc *doc;
@@ -286,19 +297,18 @@ get_xml_file (BugzillaBTS *bts, const char *key, XMLFunc parse_func)
 	xmlfile = g_new0 (BugzillaXMLFile, 1);
 	xmlfile->xml_func = parse_func;
 
-	xmlfile->system_path = g_strdup_printf (BUDDY_DATADIR "/bugzilla/%s/%s", bts->subdir, key);
-	tmppath = gnome_util_home_file ("bug-buddy.d/bugzilla/");
-	localdir = g_concat_dir_and_file (tmppath, bts->subdir);
-	xmlfile->cache_path = g_concat_dir_and_file (localdir, key);
+	tmppath = gnome_util_home_file ("bug-buddy.d");
+
+	xmlfile->system_path = g_build_filename (BUDDY_DATADIR, "bugzilla", bts->subdir, key, NULL);
+	xmlfile->cache_path  = g_build_filename (tmppath,       "bugzilla", bts->subdir, key, NULL);
+	localdir = g_build_filename (tmppath, "bugzilla", bts->subdir, NULL);
 
 	g_free (tmppath);
 
 	if (stat (xmlfile->cache_path, &local_stat)) {
-		err = g_strerror (errno);
-		d(g_message ("could not stat local file: `%s': %s\n", xmlfile->cache_path, err));
+		d(g_message ("could not stat local file: `%s': %s\n", xmlfile->cache_path, g_strerror (errno)));
 		if (e_mkdir_hier (localdir, S_IRWXU)) {
-			err = g_strerror (errno);
-			d(g_warning ("could not create local dir: `%s': %s\n", localdir, err));
+			d(g_warning ("could not create local dir: `%s': %s\n", localdir, g_strerror (errno)));
 			g_free (localdir);			
 			return xmlfile;
 		}
@@ -309,8 +319,7 @@ get_xml_file (BugzillaBTS *bts, const char *key, XMLFunc parse_func)
 	g_free (localdir);
 
 	if (stat (xmlfile->system_path, &sys_stat)) {
-		err = g_strerror (errno);
-		d(g_warning ("could not stat sys file: `%s': %s\n", xmlfile->system_path, err));
+		d(g_warning ("could not stat sys file: `%s': %s\n", xmlfile->system_path, g_strerror (errno)));
 		goto append_uris;
 	}
        
@@ -376,8 +385,8 @@ load_bugzilla (const char *filename)
 	BugzillaBTS *bts;
 	char *path, *pixmap;
 	gboolean def;
-	GdkPixbuf *pb;
 	char *s;
+	GdkPixbuf *pb;
 	
 	d(g_print ("loading `%s'...\n", filename));
 
@@ -403,7 +412,7 @@ load_bugzilla (const char *filename)
 	if (pixmap[0] == '/')
 		bts->icon = g_strdup (pixmap);
 	else
-		bts->icon = g_concat_dir_and_file (BUDDY_DATADIR, pixmap);
+		bts->icon = g_build_filename (BUDDY_DATADIR, pixmap, NULL);
 	g_free (pixmap);
 
 	s = gnome_config_get_string ("submit_type=freitag");
@@ -417,12 +426,10 @@ load_bugzilla (const char *filename)
 
 	bts->email = gnome_config_get_string ("email");
 
-	pb = gdk_pixbuf_new_from_file (bts->icon);
+	pb = gdk_pixbuf_new_from_file (bts->icon, NULL);
 	if (pb) {
-		GdkPixbuf *pb2 = gdk_pixbuf_scale_simple (pb, CLIST_HEIGHT, CLIST_HEIGHT, GDK_INTERP_BILINEAR);
-		gdk_pixbuf_render_pixmap_and_mask (pb2, &bts->pixmap, &bts->mask, 127);
+		bts->pixbuf = gdk_pixbuf_scale_simple (pb, 20, 20, GDK_INTERP_HYPER);
 		gdk_pixbuf_unref (pb);
-		gdk_pixbuf_unref (pb2);
 	}
 
 	bts->severity_node   = gnome_config_get_string ("severity_node=severities");
@@ -441,49 +448,6 @@ load_bugzilla (const char *filename)
 	return bts;
 }
 
-static int
-start_xfer (gpointer null)
-{
-	/*gtk_widget_show (GET_WIDGET ("progress-cancel"));*/
-
-	while (gtk_events_pending ())
-		gtk_main_iteration ();
-
-	if (GNOME_VFS_OK != gnome_vfs_async_xfer (	    
-		    &druid_data.vfshandle,
-		    druid_data.dlsources,
-		    druid_data.dldests,
-		    GNOME_VFS_XFER_DEFAULT,
-		    GNOME_VFS_XFER_ERROR_MODE_ABORT,
-		    GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
-		    async_update, NULL, NULL, NULL))
-		goto_product_page ();
-
-	return FALSE;
-}
-
-#if 0
-/* 
- * GNOME VFS has no way to pause the progress, which is what we want to do, really 
- * so just let it cancel for now 
- */
-static gint
-check_yoself (void)
-{
-	GtkWidget *m;
-
-	m = gnome_question_dialog (
-		_("Are you sure you want to cancel this update?"), NULL, NULL);
-	
-	if (gnome_dialog_run_and_close (GNOME_DIALOG (m))) {
-		gnome_vfs_async_cancel (druid_data.vfshandle);
-		goto_product_page ();
-		return TRUE;
-	}
-	return FALSE;
-}
-#endif
-
 void
 on_progress_cancel_clicked (GtkWidget *w, gpointer data)
 {
@@ -494,14 +458,6 @@ on_progress_cancel_clicked (GtkWidget *w, gpointer data)
 	d(g_print ("scooby dooby doo!\n"));
 }
 
-static void
-download_stuff (void)
-{
-	gtk_idle_add (start_xfer, NULL);
-	gtk_main ();
-	/*gtk_widget_hide (GET_WIDGET ("progress-cancel"));*/
-}
-
 static void
 show_products (GtkWidget *w, gpointer data)
 {
@@ -556,11 +512,11 @@ load_bugzilla_xml (void)
 		}
 
 		w = gtk_menu_item_new_with_label (bts->name);
-		gtk_signal_connect (GTK_OBJECT (w), "activate",
-				    GTK_SIGNAL_FUNC (show_products),
-				    bts);
+		g_signal_connect (G_OBJECT (w), "activate",
+				  G_CALLBACK (show_products),
+				  bts);
 		gtk_widget_show (w);
-		gtk_menu_append (GTK_MENU (m), w);
+		gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
 	}
 	w = GET_WIDGET ("bts-menu");
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (w), m);
@@ -568,6 +524,7 @@ load_bugzilla_xml (void)
 	bugzilla_bts_add_products_to_clist (druid_data.all_products);	
 }
 
+#if 0
 static void
 p_string (GnomeVFSURI *uri, gpointer data)
 {
@@ -577,6 +534,70 @@ p_string (GnomeVFSURI *uri, gpointer data)
 	g_print ("\t%s\n", s);
 	g_free (s);
 }
+#endif
+
+static void
+create_products_list (void)
+{
+	GtkListStore *model;
+	GtkTreeView *view;
+	GtkCellRenderer *ren;
+
+	view = GTK_TREE_VIEW (GET_WIDGET ("product-list"));
+
+	model = gtk_list_store_new (PRODUCT_COLS, GDK_TYPE_PIXBUF, 
+				    G_TYPE_STRING, G_TYPE_STRING,
+				    G_TYPE_POINTER);
+
+	g_object_set (G_OBJECT (view), "model", model, NULL);
+	g_object_unref (G_OBJECT (model));
+	
+	ren = gtk_cell_renderer_pixbuf_new ();
+	gtk_tree_view_insert_column_with_attributes (view, -1,
+						     "", ren,
+						     "pixbuf", PRODUCT_ICON,
+						     NULL);
+	ren = gtk_cell_renderer_text_new ();
+	gtk_tree_view_insert_column_with_attributes (view, -1,
+						     _("Product"), ren,
+						     "text", PRODUCT_NAME,
+						     NULL);
+
+	ren = gtk_cell_renderer_text_new ();
+	gtk_tree_view_insert_column_with_attributes (view, -1,
+						     _("Description"), ren,
+						     "text", PRODUCT_DESC,
+						     NULL);
+}
+
+static void
+create_components_list (void)
+{
+	GtkListStore *model;
+	GtkTreeView *view;
+	GtkCellRenderer *ren;
+
+	view = GTK_TREE_VIEW (GET_WIDGET ("component-list"));
+
+	model = gtk_list_store_new (COMPONENT_COLS, G_TYPE_STRING, 
+				    G_TYPE_STRING, G_TYPE_POINTER);
+	
+	g_object_set (G_OBJECT (view), "model", model, NULL);
+	g_object_unref (G_OBJECT (model));
+
+	ren = gtk_cell_renderer_text_new ();
+	gtk_tree_view_insert_column_with_attributes (view, -1,
+						     _("Component"), ren,
+						     "text", COMPONENT_NAME,
+						     NULL);
+
+	ren = gtk_cell_renderer_text_new ();
+	gtk_tree_view_insert_column_with_attributes (view, -1,
+						     _("Description"), ren,
+						     "text", COMPONENT_DESC,
+						     NULL);
+}
+
 
 void
 load_bugzillas (void)
@@ -585,20 +606,27 @@ load_bugzillas (void)
 	GtkWidget *w;
 	DIR *dir;
 	struct dirent *dent;
-	char *s, *p;
+	char *p;
 
 	dir = opendir (BUDDY_DATADIR"/bugzilla/");
 	if (!dir) {
-		s = g_strdup_printf (_("Could not open '%s'.\n"
-				       "Please make sure Bug Buddy was "
-				       "installed correctly."),
-				     BUDDY_DATADIR "/bugzilla/");
-		w = gnome_error_dialog (s);
-		g_free (s);
-		gnome_dialog_run_and_close (GNOME_DIALOG (w));
+		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+					    GTK_DIALOG_NO_SEPARATOR,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("Could not open '%s'.\n"
+					      "Please make sure Bug Buddy was "
+					      "installed correctly."),
+					    BUDDY_DATADIR "/bugzilla/");
+		gtk_dialog_run (GTK_DIALOG (w));
+		gtk_widget_destroy (w);
 		exit (0);
 	}
 
+	create_products_list ();
+
+	create_components_list ();
+
 	druid_data.all_products = g_new0 (BugzillaBTS, 1);
 	druid_data.all_products->name = _("All");
 
@@ -612,7 +640,7 @@ load_bugzillas (void)
 		if (!p || strcmp (p, ".bugzilla")) 
 			continue;	
 		
-		p = g_concat_dir_and_file (BUDDY_DATADIR"/bugzilla", dent->d_name);
+		p = g_build_filename (BUDDY_DATADIR, "bugzilla", dent->d_name, NULL);
 		d(g_print ("trying to load `%s'\n", p));
 		bts = load_bugzilla (p);
 		g_free (p);
@@ -623,19 +651,24 @@ load_bugzillas (void)
 	}
 
 	if (druid_data.need_to_download) {
-		GtkWidget *w = gnome_question_dialog (
-			_("Bug Buddy has determined that some of its information about\n"
-			  "the various bug tracking systems may need to be updated.\n\n"
-			  "Should Bug Buddy try to update these files now?"), NULL, NULL);
+		GtkWidget *w;
 
-		gnome_dialog_set_default (GNOME_DIALOG (w), GNOME_YES);
+		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+					    GTK_DIALOG_NO_SEPARATOR,
+					    GTK_MESSAGE_QUESTION,
+					    GTK_BUTTONS_YES_NO,
+					    _("Bug Buddy has determined that some of its information about\n"
+					      "the various bug tracking systems may need to be updated.\n\n"
+					      "Should Bug Buddy try to update these files now?"));
+
+		gtk_dialog_set_default_response (GTK_DIALOG (w), GTK_RESPONSE_YES);
 
 		d(g_print ("downloading:\n"));
 		d(g_list_foreach (druid_data.dlsources, (GFunc)p_string, NULL));
 		d(g_print ("to:\n"));
 		d(g_list_foreach (druid_data.dldests, (GFunc)p_string, NULL));
 
-		if (!gnome_dialog_run_and_close (GNOME_DIALOG (w)))
+		if (GTK_RESPONSE_YES != gtk_dialog_run (GTK_DIALOG (w))) {
 			if (GNOME_VFS_OK == gnome_vfs_async_xfer (	    
 				    &druid_data.vfshandle,
 				    druid_data.dlsources,
@@ -643,56 +676,60 @@ load_bugzillas (void)
 				    GNOME_VFS_XFER_DEFAULT,
 				    GNOME_VFS_XFER_ERROR_MODE_ABORT,
 				    GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
-				    async_update, NULL, NULL, NULL))
+				    GNOME_VFS_PRIORITY_DEFAULT,
+				    async_update, NULL, NULL, NULL)) {
+				gtk_widget_destroy (w);
 				return;
+			}
+		}
+		gtk_widget_destroy (w);
 	}
 	goto_product_page ();
 }
 
 static void
-add_product (BugzillaProduct *p, GtkCList *w)
+add_product (BugzillaProduct *p, GtkListStore *store)
 {
-	char *entry[3] = { NULL };
-	gint row;
+	GtkTreeIter iter;
 
-	entry[0] = p->name;
-	entry[1] = p->description;
-
-	row = gtk_clist_append (w, entry);
-	if (p->bts->pixmap)
-		gtk_clist_set_pixtext (w, row, 0, p->name, GNOME_PAD_SMALL, 
-				       p->bts->pixmap, p->bts->mask);
-
-	gtk_clist_set_row_data (w, row, p);
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter,
+			    PRODUCT_ICON, p->bts->pixbuf,
+			    PRODUCT_NAME, p->name,
+			    PRODUCT_DESC, p->description,
+			    PRODUCT_DATA, p,
+			    -1);
 }
 
 void
 bugzilla_bts_add_products_to_clist (BugzillaBTS *bts)
 {
-	GtkCList *w;
+	GtkTreeView *w;
+	GtkListStore *store;
 
-	w = GTK_CLIST (GET_WIDGET ("product_list"));
-	gtk_clist_freeze (w);
-	gtk_clist_clear (w);
-	on_product_list_unselect_row (GTK_WIDGET (w), 0, 0, NULL);
+	w = GTK_TREE_VIEW (GET_WIDGET ("product-list"));
+	gtk_tree_view_set_headers_visible (w, TRUE);
 
-	g_slist_foreach (bts->products, (GFunc)add_product, w);
-	gtk_clist_columns_autosize (w);
+	g_object_get (G_OBJECT (w), "model", &store, NULL);
 
-	gtk_clist_thaw (w);
+	gtk_list_store_clear (store);
+	druid_data.product = NULL;
+
+	g_slist_foreach (bts->products, (GFunc)add_product, store);
+	gtk_tree_view_columns_autosize (w);
 }
 
 static void
-add_component (BugzillaComponent *comp, GtkCList *w)
+add_component (BugzillaComponent *comp, GtkListStore *store)
 {
-	char *entry[3] = { NULL };
-	gint row;
+	GtkTreeIter iter;
 
-	entry[0] = comp->name;
-	entry[1] = comp->description;
-
-	row = gtk_clist_append (w, entry);
-	gtk_clist_set_row_data (w, row, comp);
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter,
+			    COMPONENT_NAME, comp->name,
+			    COMPONENT_DESC, comp->description,
+			    COMPONENT_DATA, comp,
+			    -1);
 }
 
 static void
@@ -707,11 +744,11 @@ add_severity (char *s, GtkMenu *m)
 	GtkWidget *w;
 
 	w = gtk_menu_item_new_with_label (s);
-	gtk_signal_connect (GTK_OBJECT (w), "activate",
-			    GTK_SIGNAL_FUNC (update_severity),
-			    s);
+	g_signal_connect (G_OBJECT (w), "activate",
+			  G_CALLBACK (update_severity),
+			  s);
 	gtk_widget_show (w);
-	gtk_menu_append (m, w);
+	gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
 	if (!strcasecmp (s, "normal") || !strcasecmp (s, "unknown")) {
 		gtk_menu_item_activate (GTK_MENU_ITEM (w));
 		gtk_option_menu_set_history (GTK_OPTION_MENU (GET_WIDGET ("severity-list")), 
@@ -723,20 +760,27 @@ void
 bugzilla_product_add_components_to_clist (BugzillaProduct *prod)
 {
 	GtkWidget *m, *c;
-	GtkCList *w;
+	GtkTreeView *w;
+	GtkListStore *store;
 
-	w = GTK_CLIST (GET_WIDGET ("component_list"));
-	gtk_clist_freeze (w);
-	gtk_clist_clear (w);
-	on_component_list_select_row (GTK_WIDGET (w), 0, 0, NULL);
+	w = GTK_TREE_VIEW (GET_WIDGET ("component-list"));
+	gtk_tree_view_set_headers_visible (w, TRUE);
 
-	g_slist_foreach (prod->components, (GFunc)add_component, w);
-	gtk_clist_columns_autosize (w);
+	g_object_get (G_OBJECT (w), "model", &store, NULL);
 
+	gtk_list_store_clear (store);
+	druid_data.component = NULL;
+
+	g_slist_foreach (prod->components, (GFunc)add_component, store);
+
+	gtk_tree_view_columns_autosize (w);
+
+#ifdef FIXME
 	if (w->rows == 1)
 		gtk_clist_select_row (w, 0, 0);
 
 	gtk_clist_thaw (w);
+#endif
 
 	m = gtk_menu_new ();
 	c = GET_WIDGET ("severity-list");
@@ -744,6 +788,50 @@ bugzilla_product_add_components_to_clist (BugzillaProduct *prod)
 	g_slist_foreach (prod->bts->severities, (GFunc)add_severity, m);
 }
 
+#define LINE_WIDTH 72
+static void
+append_line_width (GString *str, char *s)
+{
+	gchar *sp;
+	if (!s) return;
+
+	if (strlen (s) < LINE_WIDTH) {
+		g_string_append (str, s);
+		g_string_append_c (str, '\n');
+		return;
+	}
+
+	for (sp = s+LINE_WIDTH; sp > s && !isspace (*sp); sp--)
+		;
+
+	if (s == sp) sp = strpbrk (s+LINE_WIDTH, "\t\n ");
+       
+	if (sp)	*sp = '\0';
+
+	g_string_append (str, s);
+	g_string_append_c (str, '\n');
+
+	if (sp) append_line_width (str, sp+1);
+}
+
+static char *
+format_for_width (const char *s)
+{
+	GString *str;
+	int i;
+	char **sv, *r;
+
+	str = g_string_new (NULL);
+
+	sv = g_strsplit (s, "\n", -1);
+	for (i = 0; sv[i]; i++)
+		append_line_width (str, sv[i]);
+
+	r = str->str;
+	g_string_free (str, FALSE);
+	return r;
+}
+
 /*
  * from bugmail_help.html:
  * 
@@ -770,20 +858,27 @@ gchar *
 generate_email_text (void)
 {	
 	char *subject, *product,  *component, *version;
-	char *opsys,   *platform, *severity,  *body;
-	char *sysinfo, *debug_info, *text_file;
+	char *opsys,   *platform, *severity,  *body, *tmp_body;
+	char *debug_info;
 	char *email, *email1;
+#if 0
+	char *text_file, *sysinfo;
+#endif
 
-	subject    = gtk_editable_get_chars (GTK_EDITABLE (GET_WIDGET ("desc-subject")), 0, -1);
+	subject    = buddy_get_text ("desc-subject");
 	product    = druid_data.product->name;
 	component  = druid_data.component->name;
-	version    = gtk_editable_get_chars (GTK_EDITABLE (GET_WIDGET ("the-version-entry")), 0, -1);
+	version    = buddy_get_text ("the-version-entry");
 	opsys      = "Linux";
 	platform   = "Debian";
 	severity   = druid_data.severity ? druid_data.severity : "Normal";
 	/* sysinfo    = generate_sysinfo (); */
-	body       = gtk_editable_get_chars (GTK_EDITABLE (GET_WIDGET ("desc-text")), 0, -1);
-	debug_info = gtk_editable_get_chars (GTK_EDITABLE (GET_WIDGET ("gdb-text")), 0, -1);
+	tmp_body   = buddy_get_text ("desc-text");
+		
+	body = format_for_width (tmp_body);
+	g_free (tmp_body);
+
+	debug_info = buddy_get_text ("gdb-text");
 
 	if (druid_data.product->bts->submit_type == BUGZILLA_SUBMIT_FREITAG) {
 		email1 = g_strdup_printf (
