@@ -5,10 +5,9 @@
  *
  * Author:  jacob berkman  <jacob@bug-buddy.org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,6 +24,7 @@
 
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libxml/parser.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 typedef struct _BugzillaBTS BugzillaBTS;
 
@@ -60,8 +60,7 @@ struct _BugzillaBTS {
 	BugzillaSubmitType submit_type;
 	char *icon;
 
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
+	GdkPixbuf *pixbuf;
 
 	char *severity_node;
 	char *severity_item;
@@ -88,6 +87,21 @@ typedef struct {
 	char            *description;
 } BugzillaComponent;
 
+enum {
+	PRODUCT_ICON,
+	PRODUCT_NAME,
+	PRODUCT_DESC,
+	PRODUCT_DATA,
+	PRODUCT_COLS
+};
+
+enum {
+	COMPONENT_NAME,
+	COMPONENT_DESC,
+	COMPONENT_DATA,
+	COMPONENT_COLS
+};
+
 void load_bugzillas (void);
 void load_bugzilla_xml (void);
 
