@@ -2,12 +2,11 @@
  *
  * Copyright (C) Jacob Berkman
  *
- * Author:  Jacob Berkman  <jberkman@andrew.cmu.edu>
+ * Author:  Jacob Berkman  <jacob@bug-buddy.org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,6 +19,9 @@
  */
 
 #include <config.h>
+
+#include "bug-buddy.h"
+
 #include <gnome.h>
 
 #include <stdio.h>
@@ -28,12 +30,10 @@
 #include <signal.h>
 #include <unistd.h>
 #include <math.h>
+#include <string.h>
 
 #include <libart_lgpl/libart.h>
 
-#include "bug-buddy.h"
-#include "util.h"
-
 #define d(x)
 
 static gint
@@ -77,8 +77,8 @@ start_gdb (void)
 	case CRASH_NONE:
 		return;
 	case CRASH_DIALOG:
-		app = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("gdb-binary-entry")));
-		extra = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("gdb-pid-entry")));
+		app = buddy_get_text ("gdb-binary-entry");
+		extra = buddy_get_text ("gdb-pid-entry");
 		druid_data.app_pid = atoi (extra);
 		kill (druid_data.app_pid, SIGCONT);
 		if (druid_data.explicit_dirty ||
@@ -89,7 +89,7 @@ start_gdb (void)
 		}
 		break;
 	case CRASH_CORE:
-		extra = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("gdb-core-entry")));
+		extra = buddy_get_text ("gdb-core-entry");
 		if (druid_data.explicit_dirty ||
 		    (old_type != CRASH_CORE) ||
 		    (!old_extra || strcmp (extra, old_extra))) {
@@ -102,10 +102,10 @@ start_gdb (void)
 	}
 
 	g_free (old_extra);
-	old_extra = g_strdup (extra);
+	old_extra = extra;
 
 	g_free (old_app);
-	old_app = g_strdup (app);
+	old_app = app;
 
 	old_type = druid_data.crash_type;
 }
@@ -118,7 +118,7 @@ stop_gdb (void)
 		return;
 	}
 	
-	g_io_channel_close (druid_data.ioc);
+	g_io_channel_shutdown (druid_data.ioc, 1, NULL);
 	waitpid (druid_data.gdb_pid, NULL, 0);
 	
 	druid_data.gdb_pid = 0;
@@ -145,17 +145,21 @@ get_trace_from_core (const gchar *core_file)
 	int status;
 	FILE *f;
 
-	gdb_cmd = g_strdup_printf ("gdb --nw --batch --core=%s", core_file);
+	gdb_cmd = g_strdup_printf ("gdb --batch --core=%s", core_file);
 
 	f = popen (gdb_cmd, "r");
 	g_free (gdb_cmd);
 
 	if (!f) {
-		gchar *s = g_strdup_printf (_("Unable to process core file with gdb:\n"
+		GtkWidget *d;
+		d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+					    GTK_DIALOG_NO_SEPARATOR,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("Unable to process core file with gdb:\n"
 					      "'%s'"), core_file);
-		GtkWidget *d = gnome_error_dialog (s);
-		g_free (s);
-		gnome_dialog_run_and_close (GNOME_DIALOG (d));
+		gtk_dialog_run (GTK_DIALOG (d));
+		gtk_widget_destroy (d);
 		return;
 	}
 
@@ -177,11 +181,15 @@ get_trace_from_core (const gchar *core_file)
 	status = pclose(f);
 
 	if (!binary) {
-		gchar *s = g_strdup_printf (_("Unable to determine which binary created\n"
+		GtkWidget *d;
+		d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+					    GTK_DIALOG_NO_SEPARATOR,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("GDB was unable to determine which binary created\n"
 					      "'%s'"), core_file);
-		GtkWidget *d = gnome_error_dialog (s);
-		g_free (s);
-		gnome_dialog_run_and_close (GNOME_DIALOG (d));
+		gtk_dialog_run (GTK_DIALOG (d));
+		gtk_widget_destroy (d);
 		return;
 	}	
 
@@ -197,44 +205,51 @@ get_trace_from_core (const gchar *core_file)
 static gboolean
 handle_gdb_input (GIOChannel *ioc, GIOCondition condition, gpointer data)
 {	
-	GtkWidget *w = NULL;
+	gboolean retval = FALSE;
 	gchar buf[1024];
 	guint len;
-	
-	if (condition == G_IO_HUP) {
-		stop_gdb ();
-		return FALSE;
-	}
+	GIOStatus io_status;
 
  gdb_try_read:
-	switch (g_io_channel_read (ioc, buf, 1024, &len)) {
-	case G_IO_ERROR_NONE:
-		break;
-	case G_IO_ERROR_AGAIN:
+	io_status = g_io_channel_read_chars (ioc, buf, 1024, &len, NULL);
+
+	switch (io_status) {
+	case G_IO_STATUS_AGAIN:
 		goto gdb_try_read;
-	default:
+	case G_IO_STATUS_ERROR:
 		d(g_warning (_("Error on read... aborting")));
-		stop_gdb ();
-		return FALSE;
+		break;
+	case G_IO_STATUS_NORMAL:
+		retval = TRUE;
+		break;
+	default:
+		break;
 	}
 
-	w = GET_WIDGET ("gdb-text");
-	gtk_text_set_point (GTK_TEXT (w),
-			    gtk_text_get_length (GTK_TEXT (w)));
-	gtk_text_insert (GTK_TEXT (w), 
-			 NULL, NULL, NULL, buf, len);
+	if (len > 0) {
+		GtkTextIter end;
+		GtkTextBuffer *buffy;
+		GtkTextView *tv;
 
-	return TRUE;
+		tv = GTK_TEXT_VIEW (GET_WIDGET ("gdb-text"));
+		buffy = gtk_text_view_get_buffer (tv);
+
+		gtk_text_buffer_get_end_iter (buffy, &end);
+		gtk_text_buffer_insert (buffy, &end, buf, len);
+	}
+
+	if (!retval)
+		stop_gdb ();
+
+	return retval;
 }
 
 void
 get_trace_from_pair (const gchar *app, const gchar *extra)
 {
 	GtkWidget *d;
-	int fd;
 	char *app2;
 	char *args[] = { "gdb",
-			 "--nw",
 			 "--batch", 
 			 "--quiet",
 			 "--command=" BUDDY_DATADIR "/gdb-cmd",
@@ -243,11 +258,15 @@ get_trace_from_pair (const gchar *app, const gchar *extra)
 	args[5] = (char *)extra;
 
 	if (!args[0]) {
+		d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+					    GTK_DIALOG_NO_SEPARATOR,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("GDB could not be found on your system.\n"
+					      "Debugging information will not be obtained."));
 		d(g_message ("Path: %s", getenv ("PATH")));
-		gnome_dialog_run_and_close (
-			GNOME_DIALOG (
-				gnome_error_dialog (_("GDB could not be found on your system.\n"
-						      "Debugging information will not be obtained."))));
+		gtk_dialog_run (GTK_DIALOG (d));
+		gtk_widget_destroy (d);
 		return;
 	}
 
@@ -256,7 +275,7 @@ get_trace_from_pair (const gchar *app, const gchar *extra)
 	
 	/* FIXME: we should probably be fully expanding the link to
 	   see if it is a directory, but unix sucks and i am lazy */
-	if (g_file_test (app, G_FILE_TEST_ISFILE | G_FILE_TEST_ISLINK))
+	if (g_file_test (app, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
 		app2 = g_strdup (app);
 	else
 		app2 = gnome_is_program_in_path (app);
@@ -270,28 +289,41 @@ get_trace_from_pair (const gchar *app, const gchar *extra)
 
 	d(g_message ("About to debug '%s'", app2));
 	
-	if (!g_file_exists (BUDDY_DATADIR "/gdb-cmd")) {
-		d = gnome_error_dialog (_("Could not find the gdb-cmd file.\n"
-					  "Please try reinstalling Bug Buddy."));
-		gnome_dialog_run_and_close (GNOME_DIALOG (d));
+	if (!g_file_test (BUDDY_DATADIR "/gdb-cmd", G_FILE_TEST_EXISTS)) {
+		d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+					    GTK_DIALOG_NO_SEPARATOR,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("Could not find the gdb-cmd file.\n"
+					      "Please try reinstalling Bug Buddy."));
+		gtk_dialog_run (GTK_DIALOG (d));
+		gtk_widget_destroy (d);
 		g_free (app2);
 		return;
 	}
 
-	druid_data.gdb_pid = start_commandv ((const char **)args, &fd);
-	if (druid_data.gdb_pid == -1) {
-		d = gnome_error_dialog (_("Error on fork()."));
-		gnome_dialog_run_and_close (GNOME_DIALOG (d));
+	/* FIXME: use GError */
+	if (!g_spawn_async_with_pipes (NULL, args, NULL, 0, NULL, NULL,
+				       &druid_data.gdb_pid,
+				       NULL, 
+				       &druid_data.fd, 
+				       NULL, NULL)) {
+		d = gtk_message_dialog_new (GTK_WINDOW (GTK_WIDGET ("druid-window")),
+					    GTK_DIALOG_NO_SEPARATOR,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("There was an error running gdb."));
+		gtk_dialog_run (GTK_DIALOG (d));
+		gtk_widget_destroy (d);
 		g_free (app2);
 		return;
 	}
 	
-	druid_data.fd = fd;
-	druid_data.ioc = g_io_channel_unix_new (fd);
-	g_io_add_watch (druid_data.ioc, G_IO_IN | G_IO_HUP, 
+	druid_data.ioc = g_io_channel_unix_new (druid_data.fd);
+	g_io_add_watch (druid_data.ioc, G_IO_IN | G_IO_HUP,
 			handle_gdb_input, NULL);
 	g_io_channel_unref (druid_data.ioc);
-	gtk_editable_delete_text (GTK_EDITABLE (GET_WIDGET ("gdb-text")), 0, -1);
+	buddy_set_text ("gdb-text", NULL);
 
 	druid_set_sensitive (FALSE, FALSE, TRUE);
 	start_animation ();
@@ -303,9 +335,3 @@ get_trace_from_pair (const gchar *app, const gchar *extra)
 
 	g_free (app2);
 }
-
-
-
-
-
-
