@@ -4,10 +4,9 @@
  *
  * Author:  Jacob Berkman  <jacob@bug-buddy.org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,11 +19,19 @@
  */
 
 #include "config.h"
-#include <gnome.h>
+
 #include "bug-buddy.h"
 
 #include "libglade-buddy.h"
 
+#include <gnome.h>
+#include <string.h>
+
+#if 0
+#include <libgnomevfs/gnome-vfs-mime.h>
+#endif
+
+#if 0
 static char *help_pages[] = {
 	"index.html",
 	"debug-info.html",
@@ -37,6 +44,7 @@ static char *help_pages[] = {
 	"summary.html",
 	NULL
 };
+#endif
 
 static char *state_title[] = {
 	N_("Welcome to Bug Buddy"),
@@ -64,42 +72,53 @@ druid_set_sensitive (gboolean prev, gboolean next, gboolean cancel)
 void
 on_druid_help_clicked (GtkWidget *w, gpointer data)
 {
+#ifdef FIXME
 	GnomeHelpMenuEntry help_entry = { "bug-buddy", NULL };
 	help_entry.path = help_pages[druid_data.state];
 	gnome_help_display (NULL, &help_entry);
+#endif
 }
 
 void
 on_druid_about_clicked (GtkWidget *button, gpointer data)
 {
-	static GtkWidget *about, *href;
+	static GtkWidget *about;
 	static const char *authors[] = {
 		"Jacob Berkman  <jacob@bug-buddy.org>",
 		NULL
 	};
 
+	static const char *documentors[] = {
+		"Telsa Gwynne  <hobbit@aloss.ukuu.org.uk>",
+		NULL
+	};
+
 	if (about) {
 		gdk_window_show (about->window);
 		gdk_window_raise (about->window);
 		return;
 	}
 		
-	about = gnome_about_new (_(PACKAGE), VERSION,
+	about = gnome_about_new (_("Bug Buddy"), VERSION,
+				 "Copyright (C) 1999, 2000, 2001 Jacob Berkman\n"
+				 "Copyright 2000, 2001 Ximian, Inc.",
 				 _("The graphical bug reporting tool for GNOME."),
 				 authors,
-				 "Copyright (C) 1999 - 2001 Jacob Berkman\n"
-				 "Copyright 2000, 2001 Ximian, Inc.",
-				 BUDDY_ICONDIR"/bug-buddy.png");
-	gtk_signal_connect (GTK_OBJECT (about), "destroy",
-			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-			    &about);
+				 documentors,
+				 NULL, NULL);
 
+	g_signal_connect (G_OBJECT (about), "destroy",
+			  G_CALLBACK (gtk_widget_destroyed),
+			  &about);
+
+#if 0
 	href = gnome_href_new ("http://bug-buddy.org/",
 			       _("The lame Bug Buddy web page"));
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about)->vbox),
 			    href, FALSE, FALSE, 0);
-
 	gtk_widget_show (href);
+#endif
+
 	gtk_widget_show (about);
 }
 
@@ -110,7 +129,6 @@ druid_set_state (BuddyState state)
 	BuddyState oldstate;
 	GtkWidget *w;
 	char *s;
-	int pos = 0;
 
 	g_return_if_fail (state >= 0);
 	g_return_if_fail (state < STATE_LAST);
@@ -133,8 +151,8 @@ druid_set_state (BuddyState state)
 			       "text", _(state_title[state]),
 			       NULL);
 
-	gtk_notebook_set_page (GTK_NOTEBOOK (GET_WIDGET ("druid-notebook")),
-			       state);	
+	gtk_notebook_set_current_page (GTK_NOTEBOOK (GET_WIDGET ("druid-notebook")),
+				       state);	
 
 	switch (druid_data.state) {
 	case STATE_INTRO:
@@ -163,10 +181,8 @@ druid_set_state (BuddyState state)
 #if 0
 		if (!druid_data.package_name)
 			determine_our_package ();
-		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("bts-package-entry")),
-				    druid_data.package_name
-				    ? druid_data.package_name
-				    : "");
+		buddy_set_text ("bts-package-entry".
+				druid_data.package_name);
 #endif
 		load_bugzilla_xml ();
 #if 0
@@ -192,8 +208,7 @@ druid_set_state (BuddyState state)
 		/* fill in the content text */
 		s = generate_email_text ();
 		w = GET_WIDGET ("email-text");
-		gtk_editable_delete_text (GTK_EDITABLE (w), 0, -1);
-		gtk_editable_insert_text (GTK_EDITABLE (w), s, strlen (s), &pos);
+		buddy_set_text ("email-text", s);
 		g_free (s);
 		break;
 	case STATE_FINISHED:
@@ -232,23 +247,23 @@ email_is_valid (const char *addy)
 		return TRUE ;
 	}
 
-	if (g_strncasecmp (rev, "moc.", 4) &&
-	    g_strncasecmp (rev, "gro.", 4) &&
-	    g_strncasecmp (rev, "ten.", 4) &&
-	    g_strncasecmp (rev, "ude.", 4) &&
-	    g_strncasecmp (rev, "lim.", 4) &&
-	    g_strncasecmp (rev, "vog.", 4) &&
-	    g_strncasecmp (rev, "tni.", 4) &&
-	    g_strncasecmp (rev, "apra.", 5) &&
+	if (g_ascii_strncasecmp (rev, "moc.", 4) &&
+	    g_ascii_strncasecmp (rev, "gro.", 4) &&
+	    g_ascii_strncasecmp (rev, "ten.", 4) &&
+	    g_ascii_strncasecmp (rev, "ude.", 4) &&
+	    g_ascii_strncasecmp (rev, "lim.", 4) &&
+	    g_ascii_strncasecmp (rev, "vog.", 4) &&
+	    g_ascii_strncasecmp (rev, "tni.", 4) &&
+	    g_ascii_strncasecmp (rev, "apra.", 5) &&
 
 	    /* In the year 2000, seven new toplevel domains were approved by ICANN. */
-	    g_strncasecmp (rev, "orea.", 5) &&
-	    g_strncasecmp (rev, "zib.", 4) &&
-	    g_strncasecmp (rev, "pooc.", 5) &&
-	    g_strncasecmp (rev, "ofni.", 5) &&
-	    g_strncasecmp (rev, "muesum.", 5) &&
-	    g_strncasecmp (rev, "eman.", 5) &&
-	    g_strncasecmp (rev, "orp.", 5)) {
+	    g_ascii_strncasecmp (rev, "orea.", 5) &&
+	    g_ascii_strncasecmp (rev, "zib.", 4) &&
+	    g_ascii_strncasecmp (rev, "pooc.", 5) &&
+	    g_ascii_strncasecmp (rev, "ofni.", 5) &&
+	    g_ascii_strncasecmp (rev, "muesum.", 5) &&
+	    g_ascii_strncasecmp (rev, "eman.", 5) &&
+	    g_ascii_strncasecmp (rev, "orp.", 5)) {
 		g_free (rev);
 		return FALSE;
 	}
@@ -265,47 +280,57 @@ intro_page_ok (void)
 	GtkWidget *w;
 	gchar *s;
 
-	w = GET_WIDGET ("email-name-entry");
-	s = gtk_entry_get_text (GTK_ENTRY (w));
+	s = buddy_get_text ("email-name-entry");
 	if (! (s && strlen (s))) {
-		w = gnome_message_box_new (_("Please enter your name."),
-					   GNOME_MESSAGE_BOX_ERROR,
-					   GNOME_STOCK_BUTTON_OK,
-					   NULL);
-		gnome_dialog_run_and_close (GNOME_DIALOG (w));
+		g_free (s);
+		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+					    GTK_DIALOG_NO_SEPARATOR,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("Please enter your name."));
+		gtk_dialog_run (GTK_DIALOG (w));
+		gtk_widget_destroy (w);
 		return FALSE;
 	}
+	g_free (s);
 
-	w = GET_WIDGET ("email-email-entry");
-	s = gtk_entry_get_text (GTK_ENTRY (w));
+	s = buddy_get_text ("email-email-entry");
 	if (!email_is_valid (s)) {
-		w = gnome_message_box_new (_("Please enter a valid email address."),
-					   GNOME_MESSAGE_BOX_ERROR,
-					   GNOME_STOCK_BUTTON_OK,
-					   NULL);
-		gnome_dialog_run_and_close (GNOME_DIALOG (w));
+		g_free (s);
+		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+					    GTK_DIALOG_NO_SEPARATOR,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("Please enter a valid email address."));
+		gtk_dialog_run (GTK_DIALOG (w));
+		gtk_widget_destroy (w);
 		return FALSE;
 	}
+	g_free (s);
 
-	w = GET_WIDGET ("email-sendmail-entry");
-	s = gtk_entry_get_text (GTK_ENTRY (w));
-	if (! (s && strlen (s) && g_file_exists (s))) {
+	s = buddy_get_text ("email-sendmail-entry");
+	if (!g_file_test (s, G_FILE_TEST_EXISTS)) {
 		if (druid_data.submit_type == SUBMIT_TO_SELF ||
 		    druid_data.submit_type == SUBMIT_REPORT) {
-			GtkWidget *d;
-			char *m;
-			m = g_strdup_printf (_("'%s' doesn't seem to exist.\n\n"
-					       "You won't be able to actually "
-					       "submit a bug report, but you will\n"
-					       "be able to save it to a file.\n\n"
-					       "Specify a new location for sendmail?"),
-					     s);	
-			d = gnome_question_dialog (m, NULL, NULL);
-			g_free (m);
-			if (GNOME_YES == gnome_dialog_run_and_close (GNOME_DIALOG (d)))
+			w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+						    GTK_DIALOG_NO_SEPARATOR,
+						    GTK_MESSAGE_QUESTION,
+						    GTK_BUTTONS_YES_NO,
+						    _("'%s' doesn't seem to exist.\n\n"
+						      "You won't be able to actually "
+						      "submit a bug report, but you will\n"
+						      "be able to save it to a file.\n\n"
+						      "Specify a new location for sendmail?"),
+						    s);	
+			if (GTK_RESPONSE_YES == gtk_dialog_run (GTK_DIALOG (w))) {
+				g_free (s);
+				gtk_widget_destroy (w);
 				return FALSE;
+			}
+			gtk_widget_destroy (w);
 		}
 	}
+	g_free (s);
 
 	return TRUE;
 }
@@ -335,95 +360,134 @@ desc_page_ok (void)
 {
 	GtkWidget *w;
 
-	char *s = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("desc-file-entry")));
+	char *s = buddy_get_text ("desc-file-entry");
 
-	if (getenv ("BUG_ME_HARDER"))
+	if (getenv ("BUG_ME_HARDER")) {
+		g_free (s);
 		return TRUE;
+	}
 
 	if (s && *s) {
 		const char *mime_type;
-		if (!g_file_exists (s)) {
-			w = gnome_error_dialog (
-				_("The specified file does not exist."));
-			gnome_dialog_run_and_close (GNOME_DIALOG (w));
+		if (!g_file_test (s, G_FILE_TEST_EXISTS)) {
+			w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+						    GTK_DIALOG_NO_SEPARATOR,
+						    GTK_MESSAGE_ERROR,
+						    GTK_BUTTONS_OK,
+						    _("The specified file does not exist."));
+			gtk_dialog_run (GTK_DIALOG (w));
+			gtk_widget_destroy (w);
+			g_free (s);
 			return FALSE;
 		}
-
-		mime_type = gnome_mime_type_of_file (s);
+#if 0
+		mime_type = gnome_vfs_get_file_mime_type (s, NULL, FALSE);
 		d(g_message (_("File is of type: %s"), mime_type));
 		
-		if (!mime_type || strncmp ("text/", mime_type, 5)) {
-			char *msg = g_strdup_printf (_("'%s' does not look like a text file."), s);
-			gnome_dialog_run_and_close (
-				GNOME_DIALOG (gnome_error_dialog (msg)));
-			g_free (msg);
+		if (strncmp ("text/", mime_type, 5)) {
+			w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+						    GTK_DIALOG_NO_SEPARATOR,
+						    GTK_MESSAGE_ERROR,
+						    GTK_BUTTONS_OK,
+						    _("'%s' is a %s file.\n\n"
+						      "Bug Buddy can only submit plaintext (text/*) files."),
+						    s, mime_type);
+			gtk_dialog_run (GTK_DIALOG (w));
+			gtk_widget_destroy (w);
+			g_free (s);
 			return FALSE;
 		}
+#endif
 	}
-	
-	s = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("desc-subject")));
-	if (!text_is_sensical (s, 6)) {
-		w = gnome_error_dialog (
-			_("You must include a comprehensible subject line in your bug report."));
-		gnome_dialog_run_and_close (GNOME_DIALOG (w));
-		return FALSE;
-	}
-	
-	s = gtk_editable_get_chars (GTK_EDITABLE (GET_WIDGET ("desc-text")), 0, -1);
-	if (!text_is_sensical (s, 8)) {
-		w = gnome_error_dialog (
-			_("You must include a comprehensible description in your bug report."));		
-		gnome_dialog_run_and_close (GNOME_DIALOG (w));
-		g_free (s);
-		return FALSE;
-	}
+	g_free (s);
 
+	s = buddy_get_text ("desc-subject");
+	if (!text_is_sensical (s, 6)) {
+		g_free (s);
+		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+					    GTK_DIALOG_NO_SEPARATOR,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("You must include a comprehensible subject line in your bug report."));
+		gtk_dialog_run (GTK_DIALOG (w));
+		gtk_widget_destroy (w);
+		return FALSE;
+	}
+	g_free (s);
+
+	s = buddy_get_text ("desc-text");
+	if (!text_is_sensical (s, 8)) {
+		g_free (s);
+		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+					    GTK_DIALOG_NO_SEPARATOR,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("You must include a comprehensible description in your bug report."));
+		gtk_dialog_run (GTK_DIALOG (w));
+		gtk_widget_destroy (w);
+		return FALSE;
+	}
+	g_free (s);
 	return TRUE;
 }
 
 static gboolean
 submit_ok (void)
 {
-	gchar *to, *s, *s2, *file=NULL, *command;
+	gchar *to, *s, *file=NULL, *command;
 	GtkWidget *w;
 	FILE *fp;
 
 	if (druid_data.submit_type != SUBMIT_FILE) {
-		w = gnome_question_dialog (_("Submit this bug report now?"),
-					   NULL, NULL);
-		if (gnome_dialog_run_and_close (GNOME_DIALOG (w)))
-			return FALSE;
-	}
-
-	w = GET_WIDGET ( (druid_data.submit_type == SUBMIT_TO_SELF)
-			 ? "email-email-entry" : "email-to-entry");
-	to = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
-
-	if (druid_data.submit_type == SUBMIT_FILE) {
-		file = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("email-file-entry")));
-		fp = fopen (file, "w");
-		if (!fp) {
-			s = g_strdup_printf (_("Unable to open file: '%s'"), file);
-			w = gnome_error_dialog (s);
-			g_free (s);
-			g_free (to);
-			gnome_dialog_run_and_close (GNOME_DIALOG (w));
+		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+					    GTK_DIALOG_NO_SEPARATOR,
+					    GTK_MESSAGE_QUESTION,
+					    GTK_BUTTONS_YES_NO,
+					    _("Submit this bug report now?"));
+		if (GTK_RESPONSE_YES != gtk_dialog_run (GTK_DIALOG (w))) {
+			gtk_widget_destroy (w);
 			return FALSE;
 		}
+		gtk_widget_destroy (w);
+	}
+
+	to = buddy_get_text (druid_data.submit_type == SUBMIT_TO_SELF 
+			     ? "email-email-entry"
+			     : "email-to-entry");
+
+	if (druid_data.submit_type == SUBMIT_FILE) {
+		file = buddy_get_text ("email-file-entry");
+		fp = fopen (file, "w");
+		if (!fp) {
+			w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+						    GTK_DIALOG_NO_SEPARATOR,
+						    GTK_MESSAGE_ERROR,
+						    GTK_BUTTONS_OK,
+						    _("Unable to open file '%s':\n%s"), 
+						    file, g_strerror (errno));
+			g_free (file);
+			g_free (to);
+			gtk_dialog_run (GTK_DIALOG (w));
+			gtk_widget_destroy (w);
+			return FALSE;
+		}
+		g_free (file);
 	} else {
-		w = GET_WIDGET ("email-sendmail-entry");
-		s = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
+		s = buddy_get_text ("email-sendmail-entry");
 		command = g_strdup_printf ("%s -i -t", s);
 
 		d(g_message (_("about to run '%s'"), command));
 		fp =  popen (command, "w");
 		g_free (command);
 		if (!fp) {
-			s2 = g_strdup_printf (_("Unable to start mail program: '%s'"), s);
-			w = gnome_error_dialog (s2);
-			g_free (s);
-			g_free (s2);
-			gnome_dialog_run_and_close (GNOME_DIALOG (w));
+			w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+						    GTK_DIALOG_NO_SEPARATOR,
+						    GTK_MESSAGE_ERROR,
+						    GTK_BUTTONS_OK,
+						    _("Unable to start mail program '%s':\n%s"), 
+						    s, g_strerror (errno));
+			gtk_dialog_run (GTK_DIALOG (w));
+			gtk_widget_destroy (w);
 			return FALSE;
 		}
 		g_free (s);
@@ -432,12 +496,8 @@ submit_ok (void)
 	{
 		char *name, *from;
 		
-		w = GET_WIDGET ("email-name-entry");
-		name = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
-
-
-		w = GET_WIDGET ("email-email-entry");
-		from = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
+		name = buddy_get_text ("email-name-entry");
+		from = buddy_get_text ("email-email-entry");
 
 		fprintf (fp, "From: %s <%s>\n", name, from);
 		g_free (from);
@@ -448,21 +508,18 @@ submit_ok (void)
 
 	if (druid_data.submit_type == SUBMIT_REPORT &&
 	    GTK_TOGGLE_BUTTON (GET_WIDGET ("email-cc-toggle"))->active) {
-		w = GET_WIDGET ("email-email-entry");
-		s = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
+		s = buddy_get_text ("email-email-entry");
 		fprintf (fp, "Cc: %s\n", s);
 		g_free (s);
 	}
 	
-	w = GET_WIDGET ("email-cc-entry");
-	s = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
+	s = buddy_get_text ("email-cc-entry");
 	if (*s) fprintf (fp, "Cc: %s\n", s);
 	g_free (s);
 
 	fprintf (fp, "X-Mailer: %s %s\n", PACKAGE, VERSION);
 
-	w = GET_WIDGET ("email-text");
-	s = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
+	s = buddy_get_text ("email-text");
 	fprintf (fp, "%s", s);
 	g_free (s);
 
@@ -475,16 +532,36 @@ submit_ok (void)
 	}
 	g_free (to);
 
-	w = GET_WIDGET ("finished-label");
-	gtk_label_set_text (GTK_LABEL (w), s);
+	buddy_set_text ("finished-label", s);
 	g_free (s);
 
 	return TRUE;
 }
 
+static gpointer
+get_selected_row (const char *w, int col)
+{
+	GtkTreeView *view;
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+	gpointer retval;
+
+	view = GTK_TREE_VIEW (GET_WIDGET (w));
+	selection = gtk_tree_view_get_selection (view);
+	
+	if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
+		return NULL;
+
+	gtk_tree_model_get (gtk_tree_view_get_model (view),
+			    &iter, col, &retval, -1);
+	
+	return retval;
+}
+
 void
 on_druid_next_clicked (GtkWidget *w, gpointer data)
 {
+	GtkWidget *d;
 	BuddyState newstate;
 	char *s;
 
@@ -506,27 +583,42 @@ on_druid_next_clicked (GtkWidget *w, gpointer data)
 		break;
 	case STATE_PRODUCT:
 		/* check that the package is ok */
+		druid_data.product = get_selected_row ("product-list", PRODUCT_DATA);
 		if (!druid_data.product) {
-			gnome_dialog_run_and_close (
-				GNOME_DIALOG (gnome_error_dialog (
-						      _("You must specify a product for your bug report."))));
+			d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+						    GTK_DIALOG_NO_SEPARATOR,
+						    GTK_MESSAGE_ERROR,
+						    GTK_BUTTONS_OK,
+						    _("You must specify a product for your bug report."));
+			gtk_dialog_run (GTK_DIALOG (d));
+			gtk_widget_destroy (d);
 			return;
 		}
 		bugzilla_product_add_components_to_clist (druid_data.product);
+		buddy_set_text ("email-to-entry", druid_data.product->bts->email);
 		break;
 	case STATE_COMPONENT:
+		druid_data.component = get_selected_row ("component-list", COMPONENT_DATA);
 		if (!druid_data.component) {
-			gnome_dialog_run_and_close (
-				GNOME_DIALOG (gnome_error_dialog (
-						      _("You must specify a component for your bug report."))));
+			d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+						    GTK_DIALOG_NO_SEPARATOR,
+						    GTK_MESSAGE_ERROR,
+						    GTK_BUTTONS_OK,
+						    _("You must specify a component for your bug report."));
+			gtk_dialog_run (GTK_DIALOG (d));
+			gtk_widget_destroy (d);
 			return;
 		}
-		s = gtk_editable_get_chars (GTK_EDITABLE (GET_WIDGET ("the-version-entry")), 0, -1);
+		s = buddy_get_text ("the-version-entry");
 		if (!s[0] && !getenv ("BUG_ME_HARDER")) {
 			g_free (s);
-			gnome_dialog_run_and_close (
-				GNOME_DIALOG (gnome_error_dialog (
-						      _("You must specify a version for your bug report."))));
+			d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+						    GTK_DIALOG_NO_SEPARATOR,
+						    GTK_MESSAGE_ERROR,
+						    GTK_BUTTONS_OK,
+						    _("You must specify a version for your bug report."));
+			gtk_dialog_run (GTK_DIALOG (d));
+			gtk_widget_destroy (d);
 			return;
 		}
 		g_free (s);
@@ -554,11 +646,17 @@ on_druid_cancel_clicked (GtkWidget *w, gpointer data)
 {
 	GtkWidget *d;
 
-	d = gnome_question_dialog (
-		_("Are you sure you want to cancel\n"
-		  "this bug report?"), NULL, NULL);
-	if (gnome_dialog_run_and_close (GNOME_DIALOG (d)))
+	d = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
+				    GTK_DIALOG_NO_SEPARATOR,
+				    GTK_MESSAGE_QUESTION,
+				    GTK_BUTTONS_YES_NO,
+				    _("Are you sure you want to cancel\n"
+				      "this bug report?"));
+	if (GTK_RESPONSE_YES != gtk_dialog_run (GTK_DIALOG (d))) {
+		gtk_widget_destroy (d);
 		return;
+	}
+	gtk_widget_destroy (d);
 
 	save_config ();
 	gtk_main_quit ();
